function [value] = chkmesh(VERT,SIMP);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% FEMBIF = < Finite Element Methods for Bifurcation >
%% Copyright (C) 1998  Michael Holst
%%
%% This program is free software; you can redistribute it and/or modify it
%% under the terms of the GNU General Public License as published by the Free
%% Software Foundation; either version 2 of the License, or (at your option)
%% any later version.
%%
%% This program is distributed in the hope that it will be useful, but WITHOUT
%% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%% FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%% more details.
%%
%% You should have received a copy of the GNU General Public License along
%% with this program; if not, write to the Free Software Foundation, Inc.,
%% 675 Mass Ave, Cambridge, MA 02139, USA.
%%
%% Typing "help 0license" should list the GNU GPL to the screen.
%%
%% rcsid="$Id: chkmesh.m,v 1.1 1998/06/10 02:42:07 mholst Exp $"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Prototype:
%
%    [value] = chkmesh(VERT,SIMP);
%
% Purpose:
%
%    Check the mesh for correctness, including conformity.
%
% Input:
%
%    VERT   = <see "mesh.m" for description of the datastructure>
%    SIMP   = <see "mesh.m" for description of the datastructure>
% 
% Output:
%    
%    value  = 1 if no error, 0 otherwise
%
% Author:   Michael Holst 060898
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% doit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   fprintf('chkmesh: starting conformity and consistency check of mesh.\n');

%%% recover array dimensions

   [N,eight]     = size(VERT);
   [L,seventeen] = size(SIMP);
   T=3;

%%% create the structures

   [VERT,SIMP] = bldsring(VERT,SIMP);

%%% check the structures

   %%% check the vertex structures
   fprintf('chkmesh: checking vertices..');
   t0 = clock;

   vertBndD = 0;
   vertBndN = 0;
   vertInt = 0;
   for i=1:N

      %%% check id for consistency
      assert( (VERT(i,4)==i), 'chkmesh 1' );

      %%% every vertex should have at least one simplex in ring
      assert( (VERT(i,7)~=0), 'chkmesh 2' );

      %%% count up the interior points
      if (VERT(i,6)==0)
         vertInt = vertInt+1;
      end

      %%% count up the dirichlet points
      if (VERT(i,6)==1)
         vertBndD = vertBndD+1;
      end

      %%% count up the neumann points
      if (VERT(i,6)>=2)
         vertBndN = vertBndN+1;
      end
   end
   the_time = etime(clock,t0);
   fprintf('..done.  [time=%g].\n', the_time);

   %%% check the simplex structures
   fprintf('chkmesh: checking simplices..');
   t0 = clock;

   bndFaceD = 0;
   bndFaceN = 0;
   for i=1:L

      %%% check id for consistency
      assert( (SIMP(i,9)==i), 'chkmesh 3' );

      %%% no simplex should be marked for refinement
      assert( (SIMP(i,12)==0), 'chkmesh 4' );
      assert( (SIMP(i,13)==0), 'chkmesh 5' );

      %%% facetypes
      nvert = SIMP(i,1:4);
      ftype = SIMP(i,5:8);

      %%% check vertex data and simplex rings for consistency
      for j=1:T
         assert( (nvert(j)~=0), 'chkmesh 10' );
         assert( (sinring(VERT,SIMP,i,nvert(j))>0), 'chkmesh 11' );
      end

      %%% check simplex rings and nabors
      for j=1:T
         if (j==1) k=2; l=3; end;
         if (j==2) k=3; l=1; end;
         if (j==3) k=2; l=1; end;

         %%% check dirichlet points on dirichlet faces
         if (ftype(j)==1)
            bndFaceD = bndFaceD+1;
            assert( (VERT(SIMP(i,k),6)==1), 'chkmesh 6' );
            assert( (VERT(SIMP(i,l),6)==1), 'chkmesh 7' );
         end

         %%% check neumann points on neumann faces
         if (ftype(j)>=2)
            bndFaceN = bndFaceN+1;
            assert( (VERT(SIMP(i,k),6)>0), 'chkmesh 8' );
            assert( (VERT(SIMP(i,l),6)>0), 'chkmesh 9' );
         end

         %%% check face nabors; verify we have a conforming mesh
         nabor = getnabor(VERT,SIMP,i,k,l);
         cond1 = ((ftype(j)>0) & (nabor==0));
         cond2 = ((ftype(j)==0) & (nabor>0));
         assert( (cond1 | cond2), 'chkmesh 12' );
      end
   end
   the_time = etime(clock,t0);
   fprintf('..done.  [time=%g]\n', the_time);

%%% destroy the structures

   [VERT,SIMP] = kilring(VERT,SIMP);

%%% return

   fprintf('chkmesh: finished conformity and consistency check of mesh.\n');
   fprintf('chkmesh:    point breakdown:      [T=%d,I=%d,D=%d,N=%d]\n', ...
      N,vertInt,vertBndD,vertBndN);
   fprintf('chkmesh:    bndry face breakdown: [D=%d,N=%d]\n', ...
      bndFaceD,bndFaceN);

   value = 1;

