function [canon] = longedge(VERT,SIMP,which);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% FEMBIF = < Finite Element Methods for Bifurcation >
%% Copyright (C) 1998  Michael Holst
%%
%% This program is free software; you can redistribute it and/or modify it
%% under the terms of the GNU General Public License as published by the Free
%% Software Foundation; either version 2 of the License, or (at your option)
%% any later version.
%%
%% This program is distributed in the hope that it will be useful, but WITHOUT
%% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%% FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%% more details.
%%
%% You should have received a copy of the GNU General Public License along
%% with this program; if not, write to the Free Software Foundation, Inc.,
%% 675 Mass Ave, Cambridge, MA 02139, USA.
%%
%% Typing "help 0license" should list the GNU GPL to the screen.
%%
%% rcsid="$Id: longedge.m,v 1.1 1998/06/10 02:42:08 mholst Exp $"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Prototype:
%
%    [canon] = longedge(VERT,SIMP,which);
%
% Purpose:
%
%    Determine the longest of the three edges represented by the possible
%    pairings of three vertices.  Use a unique integer label to break ties.
%
% Input:
%
%    which  = the simplex number to determine the longest edge of
%    VERT   = <see "mesh.m" for description of the datastructure>
%    SIMP   = <see "mesh.m" for description of the datastructure>
%
% Output:
%
%    canon = the edge number of the longest edge
%
% A useful mapping:
%
%    mapE[3][3] ==> Convenient (invertible, pairs-to-singles) edge mapping:
%
%                   Edge (vtx pair)  Edge (number)
%                   --------------   -----------
%                   (1,2) or (2,1)   1
%                   (1,3) or (3,1)   2
%                   (2,3) or (3,2)   3
%
% Author:   Michael Holst 060898
%

%%% this mapping allows us to map vertex pairs onto single edge numbers

   mapE  = [ 0,  1,  2;
             1,  0,  3;
             2,  3,  0 ];

%%% recover array dimensions

   T=3;

%%% get the element info

   xname  = zeros(T);    %%% vertex numbers of this simp
   xcoord = zeros(T,3);  %%% the x,y,z coordinates of the vertices
   for i=1:T
      xname(i)    = SIMP( which, i );
      xcoord(i,:) = VERT( xname(i), 1:3 );
   end

%%% determin the longest edge

   maxI   = -1;
   maxJ   = -1;
   maxLen = -1;
   maxId  = -1;
   for i=1:T
      xi = xcoord(i,:);
      for j=i+1:T
         xj = xcoord(j,:);

         %%% first get unique global id for tie-breaking
         myId = 1000000 * xname(i) + xname(j);

         len = norm(xi-xj);
         if ( len >= maxLen ) 
             if ( (len > maxLen) | (myId < maxId) )
                maxLen = len;
                maxId  = myId;
                maxI = i;
                maxJ = j;
             end
         end
      end
   end
   assert( (maxI >= 0), 'longedge 1' );
   assert( (maxJ >= 0), 'longedge 2' );

%%% return result

   canon = mapE(maxI,maxJ);

