function [VERT_F,SIMP_F] = mesh(key);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% FEMBIF = < Finite Element Methods for Bifurcation >
%% Copyright (C) 1998  Michael Holst
%%
%% This program is free software; you can redistribute it and/or modify it
%% under the terms of the GNU General Public License as published by the Free
%% Software Foundation; either version 2 of the License, or (at your option)
%% any later version.
%%
%% This program is distributed in the hope that it will be useful, but WITHOUT
%% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
%% FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
%% more details.
%%
%% You should have received a copy of the GNU General Public License along
%% with this program; if not, write to the Free Software Foundation, Inc.,
%% 675 Mass Ave, Cambridge, MA 02139, USA.
%%
%% Typing "help 0license" should list the GNU GPL to the screen.
%%
%% rcsid="$Id: mesh.m,v 1.1 1998/06/10 02:42:09 mholst Exp $"
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Prototype:
%
%    [VERT,SIMP] = mesh(n);
%
% Purpose:
%
%    Define a 2D or 3D simplicial finite element (coordinates always in R^3).
%
% Output:
%
%    (N = number of vertices in mesh)
%    (L = number of simplices in mesh)
%    (K = number of edges in mesh)
%
%    VERT = size(N,8) ==> vertices
%
%              VERT(:,1)  == x-coordinates of all vertices
%              VERT(:,2)  == y-coordinates of all vertices
%              VERT(:,3)  == z-coordinates of all vertices (3d case & manifold)
%              VERT(:,4)  == vertex identifier
%              VERT(:,5)  == vertex processor id or color
%              VERT(:,6)  == vertex type
%              VERT(:,7)  == blank; later used as first simplex ptr
%              VERT(:,8)  == blank; later used as first edge ptr
%
%    SIMP = size(L,17) = simplices (vertex numbers are counter-clockwise!)
%
%              SIMP(:,1)  == 1st-vertex of the simplex
%              SIMP(:,2)  == 2nd-vertex of the simplex
%              SIMP(:,3)  == 3rd-vertex of the simplex
%              SIMP(:,4)  == 4th-vertex of the simplex (3d case)
%              SIMP(:,5)  == 1st-face type; face opposite vertex 1
%              SIMP(:,6)  == 2nd-face type; face opposite vertex 2
%              SIMP(:,7)  == 3rd-face type; face opposite vertex 3
%              SIMP(:,8)  == 4th-face type; face opposite vertex 4 (3d case)
%              SIMP(:,9)  == simplex identifier
%              SIMP(:,10) == simplex processor id or color
%              SIMP(:,11) == simplex type
%              SIMP(:,12) == first queue marker
%              SIMP(:,13) == second queue marker
%              SIMP(:,14) == blank; later used as simplex ring ptr 1
%              SIMP(:,15) == blank; later used as simplex ring ptr 2
%              SIMP(:,16) == blank; later used as simplex ring ptr 3
%              SIMP(:,17) == blank; later used as simplex ring ptr 4 (3d case)
%
%    EDGE = size(K,8) ==> edges
%
%              EDGE(:,1) == 1st vertex making up edge
%              EDGE(:,2) == 2nd vertex making up edge
%              EDGE(:,3) == midpoint vertex number
%              EDGE(:,4) == edge identifier
%              EDGE(:,5) == edge processor id or color
%              EDGE(:,6) == edge type
%              EDGE(:,7) == next edge number in ring of edges about vertex 1
%              EDGE(:,8) == next edge number in ring of edges about vertex 2
%
% Author:   Michael Holst 060898
%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% doit
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% read in the mcf file

   mcf

%%% change to internal storage

   N=vertices;
   L=simplices;
   VERT_F=zeros(N,8);
   SIMP_F=zeros(L,15);
   ONEN=ones(N,1);
   ONEL=ones(L,1);

   VERT_F(:,1:3) = vert(:,4:6);
   VERT_F(:,4:6) = vert(:,1:3);
   VERT_F(:,4)   = VERT_F(:,4) + ONEN;

   if (dim == 2)
       SIMP_F(:,1:3)  = simp(:,7:9);
       SIMP_F(:,5:7)  = simp(:,4:6);
       SIMP_F(:,9:11) = simp(:,1:3);
       SIMP_F(:,1)    = SIMP_F(:,1)  + ONEL;
       SIMP_F(:,2)    = SIMP_F(:,2)  + ONEL;
       SIMP_F(:,3)    = SIMP_F(:,3)  + ONEL;
       SIMP_F(:,9)    = SIMP_F(:,9)  + ONEL;
   else
       SIMP_F(:,1:4)  = simp(:,8:11);
       SIMP_F(:,5:8)  = simp(:,4:7);
       SIMP_F(:,9:11) = simp(:,1:3);
       SIMP_F(:,1)    = SIMP_F(:,1)  + ONEL;
       SIMP_F(:,2)    = SIMP_F(:,2)  + ONEL;
       SIMP_F(:,3)    = SIMP_F(:,3)  + ONEL;
       SIMP_F(:,4)    = SIMP_F(:,4)  + ONEL;
       SIMP_F(:,9)    = SIMP_F(:,9)  + ONEL;
   end

